{******************************************************************************
 *                                                                            *
 *                                LayGO (tm)                                  *
 *           Copyright (c) 1998-2000 by Advanced Relay Corporation            *
 *                           All rights reserved.                             *
 *                                                                            *
 ******************************************************************************

  $Source: Q:/laygorcs/Device.pas $

      $Id: Device.pas 1.2 2001/04/27 19:07:29 tiktin Exp $

  EXTERNAL MODULE DESCRIPTION

    Exports Pascal Language prototypes for the LayGO Device Library.

    See the LayGO Device Library Manual for complete details on
    function arguments and return values.

 ******************************************************************************}

unit Device;

interface

uses
    Laygo;

const

    { Error codes }
    ERROR_INTERNAL = -100;
    ERROR_OUT_OF_MEMORY = -99;
    ERROR_CANNOT_OPEN_DATABASE = -98;
    ERROR_SERVICE_OPEN_FAILURE = -97;
    ERROR_DEVICE_CONSTRUCTION_FAILURE = -96;
    ERROR_ATTACH_FAILURE = -95;
    ERROR_DETACH_FAILURE = -94;
    ERROR_CONNECT_FAILURE = -93;
    ERROR_LISTEN_FAILURE = -92;
    ERROR_DISCONNECT_FAILURE = -91;
    ERROR_REOPEN_FAILURE = -90;
    ERROR_INVALID_DEVICE = -89;
    ERROR_INVALID_CID = -88;
    ERROR_INVALID_PARAMETER = -87;
    ERROR_DEVICE_NOT_FOUND = -86;
    ERROR_DEVICE_NOT_OPEN = -85;
    ERROR_DEVICE_NOT_CONNECTED = -84;
    ERROR_DEVICE_NOT_LISTENING = -83;
    ERROR_DEVICE_NOT_DISCONNECTED = -82;
    ERROR_DEVICE_NOT_BLOCKED = -81;
    ERROR_UNBLOCK_FAILURE = -80;
    ERROR_TEMPORARY_BLOCK = -79;
    ERROR_MULTIPLEX_DEVICE = -78;
    ERROR_NO_PROTOCOL_IN_DEVICE = -77;
    ERROR_MAX = -76;

type

    { Device is an opaque type }
    LDevice = Pointer;
    LDeviceName = Pchar;
    LDevicePtr = ^LDevice;
    LProtocolPtr = ^LProtocol;

{******************************************************************************
    External function declarations
 ******************************************************************************}
function dev_Attach(device: LDevice):
                LResult;
                cdecl; external 'laygo32d.dll';
function dev_Check(device: LDevice):
                LResult;
                cdecl; external 'laygo32d.dll';
function dev_Close(device: LDevice):
                LResult;
                cdecl; external 'laygo32d.dll';
function dev_Connect(device: LDevice):
                LResult;
                cdecl; external 'laygo32d.dll';
function dev_ConnectWithData(device: LDevice; buffer: LCtlBuffer; size: LBufferSize):
                LResult;
                cdecl; external 'laygo32d.dll';
function dev_Detach(device: LDevice):
                LResult;
                cdecl; external 'laygo32d.dll';
function dev_Disconnect(device: LDevice):
                LResult;
                cdecl; external 'laygo32d.dll';
function dev_Dispose(device: LDevice):
                LResult;
                cdecl; external 'laygo32d.dll';
function dev_Fix(device: LDevice):
                LResult;
                cdecl; external 'laygo32d.dll';
function dev_GetLastError(device: LDevice; protocol: LProtocolPtr):
                LResult;
                cdecl; external 'laygo32d.dll';
function dev_GetProtocolCid(device: LDevice; protocol: LProtocol):
                LCid;
                cdecl; external 'laygo32d.dll';
function dev_GetReadWriteCid(device: LDevice):
                LCid;
                cdecl; external 'laygo32d.dll';
function dev_IsConnected(device: LDevice):
                LBoolean;
                cdecl; external 'laygo32d.dll';
function dev_Listen(device: LDevice):
                LResult;
                cdecl; external 'laygo32d.dll';
function dev_ListenCheck(device: LDevice):
                LResult;
                cdecl; external 'laygo32d.dll';
function dev_New(deviceName: LDeviceName; device: LDevicePtr):
                LResult;
                cdecl; external 'laygo32d.dll';
function dev_Open(device: LDevice):
                LResult;
                cdecl; external 'laygo32d.dll';
function dev_Reopen(device: LDevice):
                LResult;
                cdecl; external 'laygo32d.dll';
function dev_Unblock(device: LDevice):
                LResult;
                cdecl; external 'laygo32d.dll';

{******************************************************************************
    Layer functions
 ******************************************************************************}
function dev_CheckLayer(cid: LCid):
                LResult;
                cdecl; external 'laygo32d.dll';
function dev_ConnectLayer(cid: LCid):
                LResult;
                cdecl; external 'laygo32d.dll';
function dev_ConnectLayerWithData(cid: LCid; buffer: LCtlBuffer; size: LBufferSize):
                LResult;
                cdecl; external 'laygo32d.dll';
function dev_DisconnectLayer(cid: LCid):
                LResult;
                cdecl; external 'laygo32d.dll';
function dev_ReconnectLayer(cid: LCid):
                LResult;
                cdecl; external 'laygo32d.dll';

{******************************************************************************
    Supervisory functions
 ******************************************************************************}
function dev_Initialize(fileName: LFileName):
                LResult;
                cdecl; external 'laygo32d.dll';

function dev_Uninitialize():
                LResult;
                cdecl; external 'laygo32d.dll';

{******************************************************************************
    Message functions
 ******************************************************************************}
function dev_ErrorMessage(errorCode: LResult):
                LMessage;
                cdecl; external 'laygo32d.dll';

implementation
{******************************************************************************
                                  End of File
 ******************************************************************************}
end.
