{******************************************************************************
 *                                                                            *
 *                                LayGO (tm)                                  *
 *           Copyright (c) 1998-2000 by Advanced Relay Corporation            *
 *                           All rights reserved.                             *
 *                                                                            *
 ******************************************************************************

  $Source: q:/laygorcs/Laygo.pas $

      $Id: Laygo.pas 1.2 2000/02/29 01:12:36 tiktin Release $

  EXTERNAL MODULE DESCRIPTION

    Exports Pascal Language prototypes for LayGO API functions.

    See the LayGO Application Programmer's Guide for complete details
    on function arguments and return values.

 ******************************************************************************}

unit Laygo;

interface

const

    lgo_INVALID_CID = -1;
    lgo_INVALID_PROTOCOL = -1;
    lgo_INVALID_EVENT = -1;
    lgo_INVALID_STATE = -2;
    lgo_INVALID_MAJOR_ID = -1;
    lgo_INVALID_MINOR_ID = -1;

    lgo_SUMMARY_MAX = 8;
    lgo_STAT_MAX = 32;

    { Error codes }
    ERROR_MIN = -52;
    ERROR_INVALID_USER_ID = -51;
    ERROR_INVALID_HARDWARE_VERSION = -50;
    ERROR_INVALID_DRIVER_VERSION = -49;
    ERROR_CLIENTS_CONNECTED = -48;
    ERROR_SERVER_NOT_CONNECTED = -47;
    ERROR_SERVER_CONNECTED = -46;
    ERROR_INVALID_CONFIGURATION = -45;
    ERROR_INVALID_PROTOCOL = -44;
    ERROR_CONFIGURATION_FILE_WRITE_ERROR = -43;
    ERROR_INVALID_DRIVER_CONFIGURATION = -42;
    ERROR_INVALID_STACK_CONFIGURATION = -41;
    ERROR_PROTOCOL_BLOCKING = -40;
    ERROR_INVALID_WRITE_DATA_SIZE = -39;
    ERROR_WRITE_QUEUE_FULL = -38;
    ERROR_SERVER_NOT_FOUND = -37;
    ERROR_SERVER_ACCESS_NOT_ENABLED = -36;
    ERROR_DRIVER_MODULE_FILE_NOT_FOUND = -35;
    ERROR_DRIVER_MODULE_FILE_READ_ERROR = -34;
    ERROR_INVALID_STACK_ID = -33;
    ERROR_INVALID_BOARD_ID = -32;
    ERROR_DRIVER_MODULE_INITIALIZATION_FAILURE = -31;
    ERROR_INVALID_DRIVER_MODULE = -30;
    ERROR_DRIVER_NOT_INSTALLED = -29;
    ERROR_HARDWARE_CONFIGURATION_CONLICT = -28;
    ERROR_INVALID_HARDWARE_CONFIGURATION = -27;
    ERROR_SPECIAL_DATA_WAITING = -26;
    ERROR_INVALID_CID = -25;
    ERROR_INVALID_COMMAND = -24;
    ERROR_INVALID_INPUT_BUFFER = -23;
    ERROR_INVALID_OUTPUT_BUFFER = -22;
    ERROR_BUFFER_SIZE = -21;
    ERROR_NO_SYSTEM_BUFFER = -20;
    ERROR_TRANSMISSION_BLOCKED = -19;
    ERROR_SERVICE_DATABASE_NOT_FOUND = -18;
    ERROR_SERVICE_UNKNOWN = -17;
    ERROR_SERVICE_NOT_AVAILABLE = -16;
    ERROR_EVENT_WAITING = -15;
    ERROR_DATA_WAITING = -14;
    ERROR_NOTHING_WAITING = -13;
    ERROR_INVALID_IOCTL_FUNCTION = -12;
    ERROR_INVALID_IOCTL_PARAMETERS = -11;
    ERROR_HARDWARE_INITIALIZATION_FAILURE = -10;
    ERROR_BUFFER_POOL_INITIALIZATION_FAILURE = -9;
    ERROR_STACK_INITIALIZATION_FAILURE = -8;
    ERROR_OUT_OF_CONNECTIONS = -7;
    ERROR_OUT_OF_MEMORY = -6;
    ERROR_CONFIGURATION_FILE_NOT_FOUND = -5;
    ERROR_CONFIGURATION_FILE_READ_ERROR = -4;
    ERROR_UNKNOWN_PROTOCOL = -3;
    ERROR_CONFIGURATION_NOT_DEFINED = -2;
    ERROR_CONFIGURATION_NOT_FOUND = -1;
    ERROR_MAX = 0;

    { Event codes }
    EVENT_CONNECT = 0;
    EVENT_DISCONNECT = 1;
    EVENT_CONFIRM = 2;
    EVENT_LISTEN = 3;
    EVENT_ACCEPT = 4;
    EVENT_REFUSE = 5;
    EVENT_CANCEL = 6;
    EVENT_WRITE = 7;
    EVENT_XOFF = 8;
    EVENT_XON = 9;
    EVENT_RESET = 10;
    EVENT_REOPEN = 11;
    EVENT_IOCTL = 12;
    EVENT_USER_EVENT_MAX = 13;

    EVENT_CONNECT_REQUEST = 13;
    EVENT_CONNECTED = 14;
    EVENT_CONNECT_TIMEOUT = 15;
    EVENT_CONNECT_ACCEPT = 16;
    EVENT_CONNECT_REFUSE = 17;
    EVENT_DISCONNECT_REQUEST = 18;
    EVENT_DISCONNECTED = 19;
    EVENT_DISCONNECTING = 20;
    EVENT_DISCONNECT_TIMEOUT = 21;
    EVENT_DISCONNECT_CONFIRMATION = 22;
    EVENT_XOFF_REQUEST = 23;
    EVENT_XON_REQUEST = 24;
    EVENT_RESETTING = 25;
    EVENT_RESET_COMPLETED = 26;
    EVENT_RESET_TIMEOUT = 27;
    EVENT_DATA_OK = 28;
    EVENT_DATA_LENGTH_ERROR = 29;
    EVENT_DATA_CRC_ERROR = 30;
    EVENT_DATA_ABORTED_FRAME = 31;
    EVENT_PROTOCOL_MESSAGE = 32;
    EVENT_PACKET_EVENT_MAX = 33;

    EVENT_MAX = EVENT_PACKET_EVENT_MAX;
    EVENT_DATA_SPECIAL = EVENT_PACKET_EVENT_MAX;
    EVENT_NONE = ERROR_NOTHING_WAITING;

    { Protocol codes }
    PROTOCOL_SDLC_HARDWARE = 0;
    PROTOCOL_SYSTEM_BUFFER_POOL = 1;
    PROTOCOL_SYSTEM_STACK = 2;
    PROTOCOL_RESERVED_3 = 3;
    PROTOCOL_X21_BIS = 4;
    PROTOCOL_HDLC_LAPB = 5;
    PROTOCOL_X25_MAJOR_DEVICE = 6;
    PROTOCOL_X25_PVC = 7;
    PROTOCOL_X25_SVC = 8;
    PROTOCOL_RETURN = 9;
    PROTOCOL_RESERVED_10 = 10;
    PROTOCOL_FRAME_RELAY_MAJOR_DEVICE = 11;
    PROTOCOL_FRAME_RELAY_VC = 12;
    PROTOCOL_FRAME_RELAY_SVC = 13;
    PROTOCOL_LAPD_MAJOR_DEVICE = 14;
    PROTOCOL_LAPD_TEI = 15;
    PROTOCOL_RESERVED_16 = 16;
    PROTOCOL_RESERVED_17 = 17;
    PROTOCOL_MAX = 18;

    { State codes }
    STATE_CLOSED = -1;
    STATE_OPEN = 0;
    STATE_WAITING_FOR_REMOTE_ACCEPT = 1;
    STATE_LISTENING = 2;
    STATE_WAITING_FOR_LOCAL_ACCEPT = 3;
    STATE_DATA_TRANSFER_XON = 4;
    STATE_DATA_TRANSFER_XOFF = 5;
    STATE_RESETTING = 6;
    STATE_WAITING_FOR_REMOTE_CONFIRMATION = 7;
    STATE_WAITING_FOR_LOCAL_CONFIRMATION = 8;
    STATE_MAX = 9;

type

    LResult = Smallint;
    LBoolean = Smallint;
    LCount = Word;
    LByteCount = Longint;
    LCid = Smallint;
    LCidPtr = ^LCid;
    LStateCode = Smallint;
    LEventCode = Smallint;
    LService = Pchar;
    LErrorCode = Smallint;
    LProtocol = Smallint;
    LFunction = Smallint;
    LDataBuffer = Pointer;
    LDataFlags = Smallint;
    LDataFlagsPtr = ^LDataFlags;
    LCtlBuffer = Pointer;
    LBufferSize = Smallint;
    LFileName = Pchar;
    LBufferSizePtr = ^LBufferSize;
    LMessage = Pchar;
    LMessageTable = ^LMessage;
    LMessageTablePtr = ^LMessageTable;
    LDriverImage = Pchar;
    LBoardId = Smallint;
    LStackId = Smallint;
    LDriverId = Smallint;
    LMajorId = Smallint;
    LMinorId = Smallint;
    LServerId = Smallint;
    LClientId = Longint;
    LHardwareId = Smallint;
    LClientName = Pchar;
    LServerAddress = Pchar;
    LUserId = Longint;

    { Record to hold statistics returned by lgo_GetStatistics() }
    LStatTableStruct = record
        bytesSent : LByteCount;                  { number of bytes sent }
        bytesRcvd : LByteCount;                  { number of bytes received }
        statCount : LCount;                      { number of additional statistics }
        stat : array[0..lgo_STAT_MAX] of LCount; { the additional statistics }
    end;

    LStatTable = ^LStatTableStruct;

    { Summary status data for a single layer }
    LSummaryStruct = record
        protocol          : LProtocol;  { block protocol }
        major             : LMajorId;   { block major id }
        minor             : LMinorId;   { block minor id }
        state             : LStateCode; { state of the block }
        isBlocked         : LBoolean;   { block's isBlocked flag }
        isBlocking        : LBoolean;   { block's isBlocking flag }
        upQueueCount      : LCount;     { number of buffers in the up queue }
        downQueueCount    : LCount;     { number of buffers in the down queue }
        controlQueueCount : LCount;     { number of buffers in the control queue }
    end;

    LSummary = ^LSummaryStruct;

    { Record to hold status returned by lgo_GetStatus() }
    LStatusReportStruct = record
        count   : LCount;                                      { number of summaries }
        summary : array[0..lgo_SUMMARY_MAX] of LSummaryStruct; { summary of each block }
    end;

    LStatusReport = ^LStatusReportStruct;

{******************************************************************************
    External function declarations
 ******************************************************************************}

function lgo_Attach(service: LService):
                LCid;
                cdecl; external 'laygo32a.dll';
function lgo_AttachProtocol(protocol: LProtocol; majorId: LMajorId; minorId: LMinorId):
                LCid;
                cdecl; external 'laygo32a.dll';
function lgo_ClearStatistics(cid: LCid):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_Close(cid: LCid):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_ConnectAccept(cid: LCid; buffer: LCtlBuffer; size: LBufferSize):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_ConnectReject(cid: LCid; buffer: LCtlBuffer; size: LBufferSize):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_ConnectRequest(cid: LCid; buffer: LCtlBuffer; size: LBufferSize):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_Detach(cid: LCid):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_DiscardData(cid: LCid):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_DisconnectConfirm(cid: LCid; buffer: LCtlBuffer; size: LBufferSize):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_DisconnectRequest(cid: LCid; buffer: LCtlBuffer; size: LBufferSize):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_Event(cid: LCid; buffer: LCtlBuffer; size: LBufferSizePtr):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_GetConfiguration(cid: LCid; buffer: LCtlBuffer; size: LBufferSize):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_GetStatistics(cid: LCid; buffer: LCtlBuffer; size: LBufferSize):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_GetStatus(cid: LCid; buffer: LCtlBuffer; size: LBufferSize):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_Initialize(cid: LCid):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_Ioctl(cid: LCid; func: LFunction; inBuffer: LCtlBuffer; inSize: LBufferSize;
                outBuffer: LCtlBuffer; outSize: LBufferSizePtr):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_Listen(cid: LCid):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_ListenCancel(cid: LCid):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_LoadConfiguration(filename: LFileName; protocol: LProtocol;
                buffer: LCtlBuffer; size: LBufferSize):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_MaxBufferSize(cid: LCid):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_Open(service: LService):
                LCid;
                cdecl; external 'laygo32a.dll';
function lgo_OpenProtocol(protocol: LProtocol; majorId: LMajorId; minorId: LMinorId):
                LCid;
                cdecl; external 'laygo32a.dll';
function lgo_Poll(cid: LCid; size: LBufferSizePtr):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_Pop(cid: LCid):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_Protocol(cid: LCid):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_Push(stackTop: LCid; newTop: LCid):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_Read(cid: LCid; buffer: LDataBuffer; size: LBufferSize):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_ReadNext(cid: LCidPtr; buffer: LDataBuffer; size: LBufferSize):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_ReadSpecial(cid: LCid; buffer: LDataBuffer; size: LBufferSize; flags: LDataFlagsPtr):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_Reopen(cid: LCid):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_ResetRequest(cid: LCid; buffer: LCtlBuffer; size: LBufferSize):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_SetConfiguration(cid: LCid; buffer: LCtlBuffer; size: LBufferSize):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_State(cid: LCid):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_Write(cid: LCid; buffer: LDataBuffer; size: LBufferSize):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_WriteSpecial(cid: LCid; buffer: LDataBuffer; size: LBufferSize; flags: LDataFlags):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_Xoff(cid: LCid):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_Xon(cid: LCid):
                LResult;
                cdecl; external 'laygo32a.dll';

{******************************************************************************
    Supervisory functions
 ******************************************************************************}
function lgo_ActivateStack:
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_ConfigureStack(filename: LFileName):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_DisableStack:
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_EnableStack:
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_InitializeServiceDatabase(filename: LFileName):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_InitializeStack(useIsr: LBoolean; multiplier: LCount):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_SetDefaultConfiguration(protocol: LProtocol; buffer: LCtlBuffer; size: LBufferSize):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_UninitializeServiceDatabase:
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_UninitializeStack:
                LResult;
                cdecl; external 'laygo32a.dll';

{******************************************************************************
    Intelligent board support functions
 ******************************************************************************}
function lgo_InitializeHardware(driverId: LDriverId; boardId: LBoardId):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_CheckHardwareInitialization(driverId: LDriverId; boardId: LBoardId):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_DownloadDriverModule(driverId: LDriverId; boardId: LBoardId;
                filename: LFileName):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_CheckDriverModuleInitialization(driverId: LDriverId; boardId: LBoardId):
                LResult;
                cdecl; external 'laygo32a.dll';

{******************************************************************************
    Server support functions
 ******************************************************************************}
function lgo_ConnectServer(serverId: LServerId):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_ConnectServerIp(serverAddress: LServerAddress):
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_ConnectServerLocal:
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_DisconnectServer:
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_HeartbeatServer:
                LResult;
                cdecl; external 'laygo32a.dll';
function lgo_ShutdownServer(serverId: LServerId):
                LResult;
                cdecl; external 'laygo32a.dll';

{******************************************************************************
    Message functions
 ******************************************************************************}
function lgo_ErrorMessage(errorCode: LResult):
                LMessage;
                cdecl; external 'laygo32m.dll';
function lgo_EventMessage(eventCode: LResult):
                LMessage;
                cdecl; external 'laygo32m.dll';
function lgo_ProtocolMessage(protocolCode: LResult):
                LMessage;
                cdecl; external 'laygo32m.dll';
function lgo_StateMessage(stateCode: LResult):
                LMessage;
                cdecl; external 'laygo32m.dll';
function lgo_StatisticMessages(protocolCode: LResult):
                LMessageTable;
                cdecl; external 'laygo32m.dll';
function lgo_StatisticMessageTable(protocolCode: LResult; table: LMessageTablePtr):
                LResult;
                cdecl; external 'laygo32m.dll';

implementation
{******************************************************************************
                                 End of File
 ******************************************************************************}
end.
