{******************************************************************************
 *                                                                            *
 *                                LayGO (tm)                                  *
 *           Copyright (c) 1998-2000 by Advanced Relay Corporation            *
 *                           All rights reserved.                             *
 *                                                                            *
 ******************************************************************************

  $Source: q:/laygorcs/X21bis.pas $

      $Id: X21bis.pas 1.2 2000/02/29 01:11:25 tiktin Release $

  EXTERNAL MODULE DESCRIPTION

    Exports X.21 bis constants, types and routines.

 ******************************************************************************}

unit X21bis;

interface

uses
    Laygo;

const
    phys_SIGNAL_CTS = $1;
    phys_SIGNAL_DCD = $2;
    phys_SIGNAL_DSR = $4;
    phys_SIGNAL_FLG = $8;
    phys_SIGNAL_RTS = $20;
    phys_SIGNAL_DTR = $40;

    phys_CLOCK_SINK = 0;
    phys_CLOCK_SOURCE = 1;
    phys_CLOCK_SYMMETRIC = 2;
    phys_CLOCK_SINK_RX = 3;
    phys_CLOCK_SINK_TX = 4;
    phys_CLOCK_SINK_NOISE_SUPPRESS = 5;
    phys_CLOCK_NRZI_ENCODING = 6;
    phys_CLOCK_FM0_ENCODING = 7;
    phys_CLOCK_FM1_ENCODING = 8;
    phys_CLOCK_MANCHESTER_ENCODING = 9;
    phys_CLOCK_MAX = 10;

    phys_TIMER_CONNECT = 0;
    phys_TIMER_DISCONNECT = 1;
    phys_TIMER_RESET = 2;
    phys_TIMER_RECONNECT = 3;
    phys_TIMER_IDLE = 4;
    phys_TIMER_MAX = 5;

    phys_IOCTL_SWITCH_LINES = 0;
    phys_IOCTL_MODEM_CONTROL = 1;
    phys_IOCTL_GET_SIGNALS = 2;
    phys_IOCTL_MAX = 3;

    phys_SIZEOF_SIGNAL_DATA = 4;

    phys_STAT_FRAMES_SENT = 0;
    phys_STAT_FRAMES_RECEIVED = 1;
    phys_STAT_CRC_ERRORS = 2;
    phys_STAT_LENGTH_ERRORS = 3;
    phys_STAT_ABORTED_FRAMES = 4;
    phys_STAT_IDLE_TIMEOUTS = 5;
    phys_STAT_HARDWARE_ANOMALIES = 6;
    phys_STAT_HARDWARE_NO_BUFFERS = 7;
    phys_STAT_SYSTEM_NO_BUFFERS = 8;
    phys_STAT_MAX = 9;

    phys_MSG_MAX = 0;

function phys_GetSignals(cid: LCid; var signals: Cardinal; var cfg: Cardinal): Boolean;
function phys_ExtractSignals(data: Cardinal): Cardinal;
function phys_ExtractSignalCfg(data: Cardinal): Cardinal;

implementation

{******************************************************************************
 *
 *  Signal data functions
 *
 ******************************************************************************}

function phys_GetSignals(cid: LCid; var signals: Cardinal; var cfg: Cardinal): Boolean;
var
    error:  LResult;
    size:   LBufferSize;
    data:   Cardinal;
begin

    size    := phys_SIZEOF_SIGNAL_DATA;
    error   := lgo_Ioctl(cid, phys_IOCTL_GET_SIGNALS, @data, 0, @data, @size);

    if (error >= 0) and (size = phys_SIZEOF_SIGNAL_DATA) then
    begin
        signals := phys_ExtractSignals(data);
        cfg     := phys_ExtractSignalCfg(data);
        phys_GetSignals := True;
    end
    else
        phys_GetSignals := False;
end;

function phys_ExtractSignals(data: Cardinal): Cardinal;
begin
    phys_ExtractSignals := (data shr 16) and $FFFF;
end;

function phys_ExtractSignalCfg(data: Cardinal): Cardinal;
begin
    phys_ExtractSignalCfg := data and $FFFF;
end;

{******************************************************************************
                                 End of File
 ******************************************************************************}
end.
