{******************************************************************************
 *                                                                            *
 *                                LayGO (tm)                                  *
 *           Copyright (c) 1998-2000 by Advanced Relay Corporation            *
 *                           All rights reserved.                             *
 *                                                                            *
 ******************************************************************************

  $Source: q:/laygorcs/DtdrvMain.pas $

      $Id: DtdrvMain.pas 1.2 2000/02/29 01:17:14 tiktin Release $

  EXTERNAL MODULE DESCRIPTION

    Exports the main Dtdrv display.

 ******************************************************************************}

unit DtdrvMain;

interface

uses
    Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
    Menus, StdCtrls, ExtCtrls, DtdrvAbout, DtdrvText, DtdrvTdat, Laygo,
    Hex, StatusReport, Statistics, X21bis, Tdat, Using, Vtx;

const

    dtLogMax = $8000;
    dtLogNew = $4000;

type

    LogMode = ( logNONE, logMINIMAL, logNORMAL, logVERBOSE );
    TMainForm = class(TForm)
        MainMenu: TMainMenu;
        FileMenu: TMenuItem;
        OpExit: TMenuItem;
        LoggingMenu: TMenuItem;
        StatisticsMenu: TMenuItem;
        DataMenu: TMenuItem;
        HelpMenu: TMenuItem;
        OpUsing: TMenuItem;
        OpAbout: TMenuItem;
        Separator5: TMenuItem;
        OpLoadTestData: TMenuItem;
        Separator4: TMenuItem;
        OpSendTestData: TMenuItem;
        OpStatistics: TMenuItem;
        OpStatusReport: TMenuItem;
        Separator2: TMenuItem;
        OpClearStatistics: TMenuItem;
        OpLogNone: TMenuItem;
        OpLogMinimal: TMenuItem;
        OpLogNormal: TMenuItem;
        OpLogVerbose: TMenuItem;
        Separator1: TMenuItem;
        OpLogClear: TMenuItem;
        OpClearDisplay: TMenuItem;
        Separator3: TMenuItem;
        SignalsGroupBox: TGroupBox;
        LedDtr: TImage;
        LedRts: TImage;
        LedDcd: TImage;
        LedCts: TImage;
        LedDsr: TImage;
        LedFlg: TImage;
        LedDtrLabel: TLabel;
        LedRtsLabel: TLabel;
        LedDcdLabel: TLabel;
        LedCtsLabel: TLabel;
        LedDsrLabel: TLabel;
        LedFlgLabel: TLabel;
        TxGroupBox: TGroupBox;
        RxGroupBox: TGroupBox;
        SendFrameSizeGroupBox: TGroupBox;
        VtxGroupBox: TGroupBox;
        SessionLogGroupBox: TGroupBox;
        OperationsGroupBox: TGroupBox;
        OpSendFrameSize: TComboBox;
        OpVtx: TCheckBox;
        LineNumberGroupBox: TGroupBox;
        StartTimeGroupBox: TGroupBox;
        ElapsedTimeGroupBox: TGroupBox;
        StateGroupBox: TGroupBox;
        OpLineNumber: TComboBox;
        StartTime: TStaticText;
        ElapsedTime: TStaticText;
        StateText: TStaticText;
        SessionLog: TMemo;
        TxFrames: TStaticText;
        TxBytes: TStaticText;
        TxFramesLabel: TLabel;
        TxBytesLabel: TLabel;
        RxFrames: TStaticText;
        RxBytes: TStaticText;
        RxFramesLabel: TLabel;
        RxBytesLabel: TLabel;
        OpOpen: TButton;
        OpConfigure: TButton;
        OpConnect: TButton;
        OpDisconnect: TButton;
        OpListen: TButton;
        OpListenCancel: TButton;
        OpSend: TButton;
        OpFullDuplex: TButton;
        OpReopen: TButton;
        OpClose: TButton;
        ConfigureDialog: TOpenDialog;
        LoadTestDataDialog: TOpenDialog;
        LedImageRed: TImage;
        LedImageGray: TImage;
        LedImageDark: TImage;
        LedImageGreen: TImage;
        UpdateTimer: TTimer;
        procedure OpExitClick(Sender: TObject);
        procedure OnShow(Sender: TObject);
        procedure OpOpenClick(Sender: TObject);
        procedure OpConfigureClick(Sender: TObject);
        procedure OpConnectClick(Sender: TObject);
        procedure OpDisconnectClick(Sender: TObject);
        procedure OpListenClick(Sender: TObject);
        procedure OpListenCancelClick(Sender: TObject);
        procedure OpSendClick(Sender: TObject);
        procedure OpFullDuplexClick(Sender: TObject);
        procedure OpReopenClick(Sender: TObject);
        procedure OpCloseClick(Sender: TObject);
        procedure OpAboutClick(Sender: TObject);
        procedure OpUsingClick(Sender: TObject);
        procedure OpLoadTestDataClick(Sender: TObject);
        procedure OpSendTestDataClick(Sender: TObject);
        procedure OpStatisticsClick(Sender: TObject);
        procedure OpStatusReportClick(Sender: TObject);
        procedure OpClearStatisticsClick(Sender: TObject);
        procedure OpClearDisplayClick(Sender: TObject);
        procedure OpLogClearClick(Sender: TObject);
        procedure OpLogNoneClick(Sender: TObject);
        procedure OpLogMinimalClick(Sender: TObject);
        procedure OpLogNormalClick(Sender: TObject);
        procedure OpLogVerboseClick(Sender: TObject);
        procedure OpVtxClick(Sender: TObject);
        procedure OnClose(Sender: TObject; var Action: TCloseAction);
        procedure OnTimer(Sender: TObject);
        procedure OnSendFrameSizeChange(Sender: TObject);
    private
        { Private declarations }
        dtCid:          LCid;
        dtBufferMax:    LBufferSize;
        dtFrameSize:    LBufferSize;
        dtInBuffer:     HexPtr;
        dtOutBuffer:    HexPtr;
        dtUseVtx:       Boolean;
        dtIsRunning:    Boolean;
        dtFullDuplex:   Boolean;
        dtState:        LStateCode;
        dtSend:         Cardinal;
        dtStartTime:    TTimeStamp;
        dtLogMode:      LogMode;
        dtLedOff:       TPicture;
        dtLedGray:      TPicture;
        dtLedDark:      TPicture;
        dtLedOn:        TPicture;
        dtRxBytes:      Cardinal;
        dtTxBytes:      Cardinal;
        dtRxFrames:     Cardinal;
        dtTxFrames:     Cardinal;
        dtTdatCount:    Cardinal;
        dtTdatFrame:    Cardinal;
        dtSignals:      Cardinal;
        dtSignalsCfg:   Cardinal;
        procedure Log(str: string);
        procedure LogReceiveFrame(frame: LEventCode; bytes: LBufferSize);
        procedure ClearLog();
        procedure LoadLedImages();
        procedure SetAllLeds(image: TPicture);
        procedure SetLed(led: TImage; image: TPicture);
        procedure SetLogMode(mode: LogMode);
        procedure UpdateState();
        procedure UpdateSignals();
        procedure UpdateTx(size: LBufferSize);
        procedure UpdateRx(size: LBufferSize);
        procedure SignalCfgChange(cfg: Cardinal; signal: Cardinal; led: TImage);
        procedure SignalChange(change: Cardinal; signal: Cardinal; led: TImage);
        procedure ClearElapsedTime();
        procedure SetElapsedTime();
        procedure ClearStartTime();
        procedure ClearDisplay();
        procedure SetStartTime();
        procedure PostError(error: String);
        procedure PostFatalError(error: String);
        procedure PostInformation(info: String);
        function  PostQuestion(question: String): Boolean;
        procedure ProcessStart(listen: Boolean);
        procedure ProcessStop();
        procedure ProcessState();
        procedure DoFullDuplex();
        procedure ReadEvent();
        function  ReceiveFrame(): Boolean;
        function  SendFrame(): Boolean;
        function  SendTestFrame(): Boolean;
        function  SendVtxFrame(): Boolean;
        procedure ConnectNotify();
        procedure DisconnectNotify();
        procedure OpenNotify();
        procedure CloseNotify();
    public
        { Public declarations }
        procedure QueueTestFrame(i: Cardinal);
    end;

var
    MainForm: TMainForm;

implementation

{$R *.DFM}

{******************************************************************************
                                Public Operations
 ******************************************************************************}
procedure TMainForm.QueueTestFrame(i: Cardinal);
begin
    if i < dtTdatCount then
    begin
        dtTdatFrame := i;
        dtSend      := dtSend + 1;
    end;
end;

{******************************************************************************
                                Form Event Handlers
 ******************************************************************************}
procedure TMainForm.OnShow(Sender: TObject);
var
    size:   LBufferSize;
    result: LResult;
    i:      Word;
begin

    for i := 1 to ParamCount do
    begin
        result := lgo_ConfigureStack(PChar(ParamStr(i)));

        if result < 0 then
        begin
            PostFatalError(Format('Stack configuration failure "%s": %s', [ParamStr(i), lgo_ErrorMessage(result)]));
        end
    end;

    result := lgo_InitializeStack(1,1);

    if result < 0 then
    begin
        PostFatalError(Format('Stack initialization failure: %s', [lgo_ErrorMessage(result)]));
    end;

    lgo_EnableStack;

    dtCid           := lgo_INVALID_CID;
    dtUseVtx        := True;
    dtIsRunning     := False;
    dtFullDuplex    := False;
    OpVtx.Checked   := dtUseVtx;
    LoadLedImages();
    SetLogMode(logNORMAL);
    CloseNotify();
    ClearStartTime();

    for i := 0 to 31 do
    begin
        OpLineNumber.Items.Append(Format('%d', [i]));
    end;

    OpLineNumber.ItemIndex := 0;

    dtBufferMax := lgo_MaxBufferSize(dtCid);
    dtInBuffer  := AllocMem(dtBufferMax);
    dtOutBuffer := AllocMem(dtBufferMax);

    HexFill(dtOutBuffer, 0, dtBufferMax, True);

    size := 2;
    i    := 0;
    while size < dtBufferMax do
    begin
        OpSendFrameSize.Items.Append(Format('%d', [size]));
        size := size * 2;
        i    := i + 1;
    end;

    OpSendFrameSize.Items.Append(Format('%d', [dtBufferMax]));

    if dtBufferMax > 128 then
    begin
        OpSendFrameSize.ItemIndex := 6;
        dtFrameSize := 128;
    end
    else
    begin
        OpSendFrameSize.ItemIndex := i;
        dtFrameSize := dtBufferMax;
    end;

    VtxInitialize(dtBufferMax);
end;

procedure TMainForm.OnClose(Sender: TObject; var Action: TCloseAction);
begin
    if dtIsRunning then
        ProcessStop;

    if dtCid <> lgo_INVALID_CID then
    begin
        lgo_Close(dtCid);
        dtCid := lgo_INVALID_CID;
    end;

    lgo_DisableStack;
    lgo_UninitializeStack;
    Action := caFree;
end;

{******************************************************************************
                               Message Posting Operations
 ******************************************************************************}
procedure TMainForm.PostError(error: String);
begin
    Application.MessageBox(PChar(error), 'Error', MB_ICONWARNING + MB_OK);
end;

procedure TMainForm.PostFatalError(error: String);
begin
    Application.MessageBox(PChar(error), 'Dtdrv Fatal Error', MB_ICONSTOP + MB_OK);
    Application.Terminate;
end;

procedure TMainForm.PostInformation(info: String);
begin
    Application.MessageBox(PChar(info), 'Information', MB_ICONINFORMATION + MB_OK);
end;

function TMainForm.PostQuestion(question: String): Boolean;
var
    answer: Integer;
begin
    answer := Application.MessageBox(PChar(question), 'Question', MB_ICONQUESTION + MB_YESNOCANCEL);

    PostQuestion := answer = IDYES;
end;

{******************************************************************************
                               Session Log Operations
 ******************************************************************************}
procedure TMainForm.Log(str: String);
var
    s:  String;
    i:  Integer;
begin

    if dtLogMode <> logNONE then
    begin
        if Length(SessionLog.Text) + Length(str) > dtLogMax then
        begin
            s := SessionLog.text;
            Delete(s, 1, dtLogNew);
            i := Pos(#13#10, s);
            if i > 0 then
                Delete(s, 1, i + 1);
            SessionLog.text := s;
        end;

        SessionLog.Text     := SessionLog.Text + str + #13#10;
        SessionLog.SelStart := Length(SessionLog.Text);
        SendMessage (SessionLog.Handle, EM_SCROLLCARET, 0, 0);
    end;
end;

procedure TMainForm.LogReceiveFrame(frame: LEventCode; bytes: LBufferSize);
var
    error:  Integer;
    vtx:    String;
    str:    String;
begin

    if dtLogMode <> logNONE then
    begin
        case frame of
        EVENT_DATA_OK:              str := 'Good';
        EVENT_DATA_LENGTH_ERROR:    str := 'Length Error';
        EVENT_DATA_CRC_ERROR:       str := 'CRC Error';
        EVENT_DATA_ABORTED_FRAME:   str := 'Abort';
        end;

        if (frame = EVENT_DATA_OK) and dtUseVtx then
        begin
            error := VtxValidateRx(dtInBuffer, bytes);
            if error < 0 then
                vtx := Format(' [Vtx Error: %s]', [VtxErrorMessage(error)])
            else
                vtx := ' [Vtx Good]';
        end
        else
            vtx := '';

        Log(Format('> %4d %s %s', [bytes, str, vtx]));

        if dtLogMode <> logMINIMAL then
        begin
            if (dtLogMode = logNORMAL) and (bytes > 16) then
                bytes := 16;
            Log(HexFormat(dtInBuffer, bytes, 16));
        end;
    end;
end;

procedure TMainForm.ClearLog();
begin
    SessionLog.Text := '';
end;

{******************************************************************************
                                 LED Operations
 ******************************************************************************}
procedure TMainForm.LoadLedImages();
begin
    dtLedOff    := LedImageRed.Picture;
    dtLedGray   := LedImageGray.Picture;
    dtLedDark   := LedImageDark.Picture;
    dtLedOn     := LedImageGreen.Picture;
end;

procedure TMainForm.SetAllLeds(image: TPicture);
begin
    LedDtr.Picture := image;
    LedRts.Picture := image;
    LedDcd.Picture := image;
    LedCts.Picture := image;
    LedDsr.Picture := image;
    LedFlg.Picture := image;
end;

procedure TMainForm.SetLed(led: TImage; image: TPicture);
begin
    led.Picture := image;
end;

{******************************************************************************
                                  Update Operations
 ******************************************************************************}
procedure TMainForm.UpdateState();
var
    state:  LStateCode;
begin

    state := lgo_State(dtCid);

    if state <> dtState then
    begin
        dtState := state;
        StateText.Caption := lgo_StateMessage(dtState);

        if dtState = STATE_DATA_TRANSFER_XON then
            ConnectNotify()
        else if dtState = STATE_WAITING_FOR_REMOTE_CONFIRMATION then
            DisconnectNotify()
        else if dtState = STATE_OPEN then
            OpenNotify();
    end;
end;

procedure TMainForm.UpdateSignals();
var
    signals:    Cardinal;
    cfg:        Cardinal;
    change:     Cardinal;
begin

    if phys_GetSignals(dtCid, signals, cfg) then
    begin
        if cfg <> dtSignalsCfg then
        begin
            SignalCfgChange(cfg, phys_SIGNAL_CTS, LedCts);
            SignalCfgChange(cfg, phys_SIGNAL_DCD, LedDcd);
            SignalCfgChange(cfg, phys_SIGNAL_DSR, LedDsr);
            SignalCfgChange(cfg, phys_SIGNAL_FLG, LedFlg);
            SignalCfgChange(cfg, phys_SIGNAL_RTS, LedRts);
            SignalCfgChange(cfg, phys_SIGNAL_DTR, LedDtr);
            dtSignalsCfg := cfg;
        end;

        change := (dtSignals xor signals) and dtSignalsCfg;

        if change <> 0 then
        begin
            SignalChange(change, phys_SIGNAL_CTS, LedCts);
            SignalChange(change, phys_SIGNAL_DCD, LedDcd);
            SignalChange(change, phys_SIGNAL_DSR, LedDsr);
            SignalChange(change, phys_SIGNAL_FLG, LedFlg);
            SignalChange(change, phys_SIGNAL_RTS, LedRts);
            SignalChange(change, phys_SIGNAL_DTR, LedDtr);
        end;

        dtSignals := signals;
    end;
end;

procedure TMainForm.SignalCfgChange(cfg: Cardinal; signal: Cardinal; led: TImage);
begin
    if (cfg And signal) = 0 then
        SetLed(led, dtLedGray)
    else if (dtSignals And signal) = 0 then
        SetLed(led, dtLedOff);
end;

procedure TMainForm.SignalChange(change: Cardinal; signal: Cardinal; led: TImage);
begin
    if (change And signal) <> 0 then
        if (dtSignals And signal) <> 0 then
            SetLed(led, dtLedOff)
        else
            SetLed(led, dtLedOn);
end;

procedure TMainForm.UpdateTx(size: LBufferSize);
begin

    dtTxBytes   := dtTxBytes + Cardinal(size);
    dtTxFrames  := dtTxFrames + 1;

    TxBytes.caption     := Format('%d', [dtTxBytes]);
    TxFrames.caption    := Format('%d', [dtTxFrames]);
end;

procedure TMainForm.UpdateRx(size: LBufferSize);
begin
    dtRxBytes   := dtRxBytes + Cardinal(size);
    dtRxFrames  := dtRxFrames + 1;

    RxBytes.Caption     := Format('%d', [dtRxBytes]);
    RxFrames.Caption    := Format('%d', [dtRxFrames]);
end;

{******************************************************************************
                            State Notify Operations
 ******************************************************************************}
procedure TMainForm.OpenNotify();
begin
    ProcessStop();

    OpOpen.Enabled              := False;
    OpConfigure.Enabled         := True;
    OpConnect.Enabled           := True;
    OpDisconnect.Enabled        := False;
    OpListen.Enabled            := True;
    OpListenCancel.Enabled      := False;
    OpSend.Enabled              := False;
    OpSendTestData.Enabled      := False;
    OpFullDuplex.Enabled        := False;
    OpReopen.Enabled            := False;
    OpClose.Enabled             := True;
    OpLineNumber.Enabled        := False;
    OpStatistics.Enabled        := True;
    OpStatusReport.Enabled      := True;
    OpClearStatistics.Enabled   := True;

    UpdateState();
    UpdateSignals();
end;

procedure TMainForm.CloseNotify();
begin

    Caption := 'Dtdrv - Closed';
    Application.Title := Caption;

    if dtCid <> lgo_INVALID_CID then
    begin
        lgo_Close(dtCid);
        dtCid := lgo_INVALID_CID;
    end;

    UpdateState();
    ClearLog();
    ClearDisplay();
    ClearElapsedTime();
    ClearStartTime();
    SetAllLeds(dtLedDark);

    OpLineNumber.Enabled        := True;
    OpOpen.Enabled              := True;
    OpConfigure.Enabled         := False;
    OpConnect.Enabled           := False;
    OpDisconnect.Enabled        := False;
    OpListen.Enabled            := False;
    OpListenCancel.Enabled      := False;
    OpSend.Enabled              := False;
    OpFullDuplex.Enabled        := False;
    OpReopen.Enabled            := False;
    OpClose.Enabled             := False;
    OpStatistics.Enabled        := False;
    OpStatusReport.Enabled      := False;
    OpClearStatistics.Enabled   := False;
    OpSendTestData.Enabled      := False;

    FocusControl(OpLineNumber);
end;

procedure TMainForm.ConnectNotify();
begin
    OpDisconnect.Enabled    := True;
    OpSend.Enabled          := True;
    OpFullDuplex.Enabled    := True;

    if dtTdatCount > 0 then
        OpSendTestData.Enabled := True;
end;

procedure TMainForm.DisconnectNotify();
begin
    OpDisconnect.Enabled    := False;
    OpSend.Enabled          := False;
    OpFullDuplex.Enabled    := False;
    OpSendTestData.Enabled  := False;
end;

{******************************************************************************
                                  Operations
 ******************************************************************************}
procedure TMainForm.OpExitClick(Sender: TObject);
begin
    Close();
end;

procedure TMainForm.OpOpenClick(Sender: TObject);
var
    majorId:    LMajorId;
    result:     LCid;
begin

    majorId := OpLineNumber.ItemIndex;

    result := lgo_OpenProtocol(PROTOCOL_X21_BIS, majorId, 0);

    if result < 0 then
        PostError(Format('Failure opening Line %d: %s',
                    [majorId, lgo_ErrorMessage(result)]))
    else
    begin

        dtCid   := result;
        Caption := Format('Dtdrv - Line %d', [majorId]);
        Application.Title := Caption;
        Log(Format('Line %d opened with CID %d', [majorId, dtCid]));

        OpenNotify();
        SetAllLeds(dtLedGray);
    end;
end;

procedure TMainForm.OpConfigureClick(Sender: TObject);
var
    result:     LResult;
    buffer:     array [0..1023] of Byte;
    cfgFile:    String;
begin

    if ConfigureDialog.Execute() then
    begin
        cfgFile := ConfigureDialog.FileName;
        result  := lgo_LoadConfiguration(PChar(cfgFile), PROTOCOL_X21_BIS, @buffer, 1024);

        if result <= 0 then
            PostError(Format('Failure loading configuration from ''%s'':' + #13#10 + '%s.',
                                [cfgFile, lgo_ErrorMessage(result)]))
        else
            if lgo_SetConfiguration(dtCid, @buffer, result) < 0 then
                PostError(Format('Failure configuring CID %d.', [dtCid]))
            else
                PostInformation(Format('CID %d configured from %s.', [dtCid, cfgFile]));
    end;
end;

procedure TMainForm.OpConnectClick(Sender: TObject);
var
    result: LResult;
begin

    result := lgo_ConnectRequest(dtCid, nil, 0);

    if result < 0 then
    begin
        PostError(Format('Connect request failure on CID %d: %s',
                    [dtCid, lgo_ErrorMessage(result)]));
    end
    else
        ProcessStart(False);
end;

procedure TMainForm.OpDisconnectClick(Sender: TObject);
var
    result: LResult;
begin

    result := lgo_DisconnectRequest(dtCid, nil, 0);

    if result < 0 then
        PostError(Format('Disconnect request failure on CID %d: %s',
                    [dtCid, lgo_ErrorMessage(result)]));
end;

procedure TMainForm.OpListenClick(Sender: TObject);
var
    result: LResult;
begin

    result := lgo_Listen(dtCid);

    if result < 0 then
        PostError(Format('Listen failure on CID %d: %s',
                    [dtCid, lgo_ErrorMessage(result)]))
    else
        ProcessStart(True);
end;

procedure TMainForm.OpListenCancelClick(Sender: TObject);
var
    result: LResult;
begin

    result := lgo_ListenCancel(dtCid);

    if result < 0 then
        PostError(Format('Listen cancel failure on CID %d: %s',
                    [dtCid, lgo_ErrorMessage(result)]))
    else
        OpenNotify();
end;

procedure TMainForm.OpSendClick(Sender: TObject);
begin
    SendFrame();
end;

procedure TMainForm.OpFullDuplexClick(Sender: TObject);
begin
    dtFullDuplex := not dtFullDuplex;

    if dtFullDuplex then
    begin
        OpDisconnect.Enabled    := False;
        OpSend.Enabled          := False;
        OpSendTestData.Enabled  := False;
        OpReopen.Enabled        := False;
    end
    else
    begin
        OpDisconnect.Enabled    := True;
        OpSend.Enabled          := True;
        OpReopen.Enabled        := True;
        if dtTdatCount > 0 then
            OpSendTestData.Enabled := True;
    end;
end;

procedure TMainForm.OpReopenClick(Sender: TObject);
var
    result: LResult;
begin

    result := lgo_Reopen(dtCid);

    if result < 0 then
        PostError(Format('Reopen failure on CID %d: %s',
                    [dtCid, lgo_ErrorMessage(result)]))
    else
        OpenNotify();
end;

procedure TMainForm.OpCloseClick(Sender: TObject);
begin
    CloseNotify();
end;

procedure TMainForm.OpAboutClick(Sender: TObject);
begin
    with TAboutForm.Create(nil) do
    try
        ShowModal();
    finally
        Free();
    end;
end;

procedure TMainForm.OpUsingClick(Sender: TObject);
begin

    TextForm.SetTitle('Using Dtdrv');
    TextForm.SetText(GetUsingText());
    TextForm.ShowModal();
end;

procedure TMainForm.OpLoadTestDataClick(Sender: TObject);
var
    load:   Boolean;
    msg:    String;
begin

    load := True;

    if dtTdatCount > 0 then
    begin
        msg := 'Test data frames are currently loaded from' + #13#10 +
                '%s.'+ #13#10 +
                'These frames will be discarded if you load' + #13#10 +
                'frames from another file.' + #13#10#13#10 +
                'Are you sure you want to load frames from' + #13#10 +
                'another file?';

        load := PostQuestion(Format(msg, [LoadTestDataDialog.FileName]));
    end;

    if load then
        if LoadTestDataDialog.Execute() then
        begin
            dtTdatCount := TdatLoadFile(LoadTestDataDialog.FileName);

            if dtTdatCount > 0 then
            begin
                PostInformation(Format('%d test frames loaded from "%s"',
                            [dtTdatCount, LoadTestDataDialog.FileName]));
                OpSendTestData.Enabled := OpSend.Enabled;
            end
            else
            begin
                PostError(Format('Failure loading test data from "%s"',
                            [LoadTestDataDialog.FileName]));
                OpSendTestData.Enabled := False;
            end;
        end;
end;

procedure TMainForm.OpSendTestDataClick(Sender: TObject);
begin
    TdatForm.ShowModal();
end;

procedure TMainForm.OpStatisticsClick(Sender: TObject);
var
    report: string;
begin

    if not GetStatisticsTable(dtCid, report) then
        PostError(report)
    else
    begin
        TextForm.SetTitle('Statistics');
        TextForm.SetText(report);
        TextForm.ShowModal();
    end;
end;

procedure TMainForm.OpStatusReportClick(Sender: TObject);
var
    report: string;
begin

    if not GetStatusReport(dtCid, report) then
        PostError(report)
    else
    begin
        TextForm.SetTitle('Status Report');
        TextForm.SetText(report);
        TextForm.ShowModal();
    end;
end;

procedure TMainForm.OpClearStatisticsClick(Sender: TObject);
var
    result: LResult;
begin

    result := lgo_ClearStatistics(dtCid);

    if result < 0 then
        PostError(Format('Failure clearing statistics on CID %d: %s',
                    [dtCid, lgo_ErrorMessage(result)]));
end;

procedure TMainForm.OpClearDisplayClick(Sender: TObject);
begin
    ClearDisplay();
end;

procedure TMainForm.ClearDisplay();
begin
    dtRxBytes           := 0;
    dtTxBytes           := 0;
    dtRxFrames          := 0;
    dtTxFrames          := 0;
    RxBytes.Caption     := '0';
    TxBytes.Caption     := '0';
    RxFrames.Caption    := '0';
    TxFrames.Caption    := '0';
end;

procedure TMainForm.OpLogClearClick(Sender: TObject);
begin
    ClearLog();
end;

procedure TMainForm.OpVtxClick(Sender: TObject);
begin
    if OpVtx.Checked then
        dtUseVtx := True
    else
        dtUseVtx := False;
end;

procedure TMainForm.OnSendFrameSizeChange(Sender: TObject);
begin
    dtFrameSize := StrToInt(OpSendFrameSize.Items.Strings[OpSendFrameSize.ItemIndex]);
end;

{******************************************************************************
                                Process Operations
 ******************************************************************************}
procedure TMainForm.ProcessStart(listen: Boolean);
begin
    OpConfigure.Enabled := False;
    OpConnect.Enabled   := False;
    OpListen.Enabled    := False;
    OpReopen.Enabled    := True;
    OpClose.Enabled     := False;
    if listen then
        OpListenCancel.Enabled := True;

    ClearDisplay();
    ClearLog();
    ClearElapsedTime();
    SetStartTime();
    UpdateState();

    dtIsRunning             := True;
    UpdateTimer.Enabled     := True;
end;

procedure TMainForm.ProcessStop();
begin
    dtIsRunning := False;
    UpdateTimer.Enabled := False;
end;

procedure TMainForm.ProcessState();
begin

    case dtState of
    STATE_WAITING_FOR_REMOTE_ACCEPT,
    STATE_LISTENING,
    STATE_RESETTING,
    STATE_WAITING_FOR_REMOTE_CONFIRMATION:

        ReadEvent();

    STATE_DATA_TRANSFER_XON:

        if dtFullDuplex then
            DoFullDuplex()
        else
        begin
            if dtSend > 0 then
                if SendFrame() then
                    dtSend := dtSend - 1;
            ReceiveFrame();
        end;

    STATE_DATA_TRANSFER_XOFF:

        ReceiveFrame();

    end;
end;

procedure TMainForm.OnTimer(Sender: TObject);
begin
    SetElapsedTime();
    ProcessState();
end;

{******************************************************************************
                             Event and Data Operations
 ******************************************************************************}
procedure TMainForm.DoFullDuplex();
var
    i: Integer;
begin

    i := 16;

    while i > 0 do
    begin
        if not ReceiveFrame() then
            i := 0
        else
            i := i - 1;
    end;

    i := 8;

    while i > 0 do
    begin
        if not SendFrame() then
            i := 0
        else
            i := i - 1;
    end;

end;

procedure TMainForm.ReadEvent();
var
    size:   LBufferSize;
    result: LResult;
begin

    size    := dtBufferMax;
    result  := lgo_Event(dtCid, dtInBuffer, @size);

    UpdateSignals();

    if result >= 0 then
    begin
        case result of
        EVENT_DATA_LENGTH_ERROR,
        EVENT_DATA_CRC_ERROR,
        EVENT_DATA_ABORTED_FRAME:
            LogReceiveFrame(result, size);
        else
            Log(Format('Event received: %s', [lgo_EventMessage(result)]));
            UpdateState();
        end;
    end;
end;

function TMainForm.ReceiveFrame(): Boolean;
var
    bytes:  LResult;
begin

    bytes := lgo_Read(dtCid, dtInBuffer, dtBufferMax);

    if bytes > 0 then
    begin
        UpdateRx(bytes);
        LogReceiveFrame(EVENT_DATA_OK, bytes);
        ReceiveFrame := True;
    end
    else if bytes = ERROR_EVENT_WAITING then
    begin
        ReadEvent();
        ReceiveFrame := False;
    end
    else
        ReceiveFrame := False;
end;

function TMainForm.SendFrame(): Boolean;
var
    bytes:  LResult;
begin

    if dtTdatFrame < dtTdatCount then
        SendFrame := SendTestFrame()
    else if dtUseVtx then
        SendFrame := SendVtxFrame()
    else
    begin
        bytes := lgo_Write(dtCid, dtOutBuffer, dtFrameSize);

        if bytes > 0 then
        begin
            UpdateTx(dtFrameSize);
            SendFrame := True;
        end
        else
            SendFrame := False;
    end;
end;

function TMainForm.SendTestFrame(): Boolean;
var
    bytes:  LResult;
    buffer: HexPtr;
    size:   LBufferSize;
begin

    buffer  := TdatGetFrame(dtTdatFrame);
    size    := TdatGetSize(dtTdatFrame);

    bytes   := lgo_Write(dtCid, buffer, size);

    if bytes > 0 then
    begin
        UpdateTx(size);
        dtTdatFrame     := dtTdatCount;
        SendTestFrame   := True;
    end
    else
        SendTestFrame   := False;
end;

function TMainForm.SendVtxFrame(): Boolean;
var
    bytes:  LResult;
    buffer: HexPtr;
begin

    buffer  := VtxGetTx(dtFrameSize);
    bytes   := lgo_Write(dtCid, buffer, dtFrameSize);

    if bytes > 0 then
    begin
        UpdateTx(dtFrameSize);
        VtxNextTx();
        SendVtxFrame := True;
    end
    else
        SendVtxFrame := False;
end;

{******************************************************************************
                              Time Display Operations
 ******************************************************************************}
procedure TMainForm.ClearElapsedTime();
begin
    ElapsedTime.Caption := '00:00:00:00';
end;

procedure TMainForm.SetElapsedTime();
var
    theTime:    TTimeStamp;
    secs:       Integer;
    days:       Integer;
    mins:       Integer;
    hrs:        Integer;
    str:        String;
begin
    theTime := DateTimeToTimeStamp(Now);

    if theTime.Date = dtStartTime.Date then
    begin
        days := 0;
        secs := (theTime.Time - dtStartTime.Time) DIV 1000;
    end
    else
    begin
        days := theTime.Date - dtStartTime.Date;
        secs := (((60 * 60 * 24 * 1000) - dtStartTime.Time) + theTime.Time) DIV 1000;
    end;

    hrs     := secs DIV 3600;
    secs    := secs MOD 3600;
    mins    := secs DIV 60;
    secs    := secs MOD 60;

    str := str + Format('%.2d:%2.2d:%2.2d:%2.2d', [days, hrs, mins, secs]);

    ElapsedTime.Caption := str;
end;

procedure TMainForm.ClearStartTime();
begin
    StartTime.Caption := '00/00/00 00:00:00';
end;

procedure TMainForm.SetStartTime();
begin
    dtStartTime         := DateTimeToTimeStamp(Now);
    StartTime.Caption   := DateTimeToStr(Now);
end;

{******************************************************************************
                              Log Mode Operations
 ******************************************************************************}
procedure TMainForm.OpLogNoneClick(Sender: TObject);
begin
    SetLogMode(logNONE);
end;

procedure TMainForm.OpLogMinimalClick(Sender: TObject);
begin
    SetLogMode(logMINIMAL);
end;

procedure TMainForm.OpLogNormalClick(Sender: TObject);
begin
    SetLogMode(logNORMAL);
end;

procedure TMainForm.OpLogVerboseClick(Sender: TObject);
begin
    SetLogMode(logVERBOSE);
end;

procedure TMainForm.SetLogMode(mode: LogMode);
begin
    case mode of
        logNONE     : OpLogNone.Checked     := True;
        logMINIMAL  : OpLogMinimal.Checked  := True;
        logNORMAL   : OpLogNormal.Checked   := True;
        logVERBOSE  : OpLogVerbose.Checked  := True;
    end;
    dtLogMode := mode;
end;

{******************************************************************************
                                 End of File
 ******************************************************************************}
end.
