{******************************************************************************
 *                                                                            *
 *                                LayGO (tm)                                  *
 *           Copyright (c) 1998-2000 by Advanced Relay Corporation            *
 *                           All rights reserved.                             *
 *                                                                            *
 ******************************************************************************

  $Source: q:/laygorcs/Hex.pas $

      $Id: Hex.pas 1.2 2000/02/29 01:17:14 tiktin Release $

  EXTERNAL MODULE DESCRIPTION

    Exports hex conversion routines.

 ******************************************************************************}

unit Hex;

interface

uses
    SysUtils;

type
    HexPtr = ^Byte;

function HexToLong(p0: HexPtr; i: Cardinal): Cardinal;
function HexToShort(p0: HexPtr; i: Cardinal): Cardinal;
function HexToBoolean(p0: HexPtr; i: Cardinal): String;
function HexFill(buffer: HexPtr; start: Cardinal; max: Cardinal; forward: Boolean): Cardinal;
function HexFormat(buffer: HexPtr; size: Cardinal; width: Cardinal): String;
function HexHeader(width: Cardinal): String;

implementation

function HexToLong(p0: HexPtr; i: Cardinal): Cardinal;
var
    p1: HexPtr;
    p2: HexPtr;
    p3: HexPtr;
begin
    {Assume little endian}
    p0 := HexPtr(Ptr(Integer(p0)+Integer(i)));
    p1 := HexPtr(Ptr(Integer(p0)+1));
    p2 := HexPtr(Ptr(Integer(p0)+2));
    p3 := HexPtr(Ptr(Integer(p0)+3));

    HexToLong :=
        (Cardinal(p3^) shl 24) + (Cardinal(p2^) shl 16) +
        (Cardinal(p1^) shl  8) +  Cardinal(p0^);
end;

function HexToShort(p0: HexPtr; i: Cardinal): Cardinal;
var
    p1: HexPtr;
begin
    {Assume little endian}
    p0 := HexPtr(Ptr(Integer(p0)+Integer(i)));
    p1 := HexPtr(Ptr(Integer(p0)+1));

    HexToShort :=
        (Cardinal(p1^) shl 8) + Cardinal(p0^);
end;

function HexToBoolean(p0: HexPtr; i: Cardinal): String;
begin
    if HexToShort(p0, i) = 0 then
        HexToBoolean := 'False'
    else
        HexToBoolean := 'True';
end;

function HexFormat(buffer: HexPtr; size: Cardinal; width: Cardinal): String;
var
    s: String;
    i: Cardinal;
    j: Cardinal;
    p: HexPtr;
begin

    i := 0;

    while i < size do
    begin
        j := i;
        p := HexPtr(Ptr(Integer(buffer)+Integer(j)));
        while (j < size) And (j < i + width) do
        begin
            s := s + Format('%3.2x', [p^]);
            j := j + 1;
            Inc(p);
        end;

        while j < i + width do
        begin
            s := s + '   ';
            j := j + 1;
        end;

        s := s + ' # ';

        j := i;
        p := HexPtr(Ptr(Integer(buffer)+Integer(j)));
        while (j < size) And (j < i + width) do
        begin
            if (p^ < 32) Or (p^ > 126) then
                s := s + '.'
            else
                s := s + Chr(p^);
            j := j + 1;
            Inc(p);
        end;

        i := i + width;

        if i < size then
            s := s + #13#10;
    end;

    HexFormat := s;
end;

function HexFill(buffer: HexPtr; start: Cardinal; max: Cardinal; forward: Boolean): Cardinal;
var
    i: Cardinal;
begin

    i := 0;

    Inc(buffer, start);
    if forward then
        while start < max do
        begin
            buffer^ := Byte(65 + (i Mod 26));
            start   := start + 1;
            i       := i + 1;
            Inc(buffer);
        end
    else
        while start < max do
        begin
            buffer^ := Byte(90 - (i Mod 26));
            start   := start + 1;
            i       := i + 1;
            Inc(buffer);
        end;

    HexFill := i;
end;

function HexHeader(width: Cardinal): String;
begin

    HexHeader := 'Hex' + StringOfChar(' ', width * 3) + 'ASCII' + #13#10 +
                            StringOfChar('-', width * 3) + ' # ' +
                            StringOfChar('-', width) + #13#10;
end;

{******************************************************************************
                                 End of File
 ******************************************************************************}
end.