{******************************************************************************
 *                                                                            *
 *                                LayGO (tm)                                  *
 *           Copyright (c) 1998-2000 by Advanced Relay Corporation            *
 *                           All rights reserved.                             *
 *                                                                            *
 ******************************************************************************

  $Source: q:/laygorcs/Statistics.pas $

      $Id: Statistics.pas 1.3 2000/03/08 20:03:43 tiktin Release $

  EXTERNAL MODULE DESCRIPTION

    Exports routines for retreiving LayGO Statistics Tables.

 ******************************************************************************}

unit Statistics;

interface

uses
    SysUtils, Laygo;

function FormatStatisticsTable(var table: LStatTableStruct; protocol: LProtocol): String;
function GetStatisticsTable(cid: LCid; var report: String): Boolean;

implementation

function GetStatisticsTable(cid: LCid; var report: String): Boolean;
var
    stats:  LStatTableStruct;
    error:  LErrorCode;
    layer:  LProtocol;
begin

    error := lgo_GetStatistics(cid, @stats, SizeOf(LStatTableStruct));

    if error < 0 then
    begin
        report := Format('Failure getting statistics table for CID %d: %s',
                            [cid, lgo_ErrorMessage(error)]);
        GetStatisticsTable := False;
    end
    else
    begin
        layer   := lgo_Protocol(cid);
        report  := Format('Statistics for %s layer (CID %d):'#13#10#13#10,
                            [lgo_ProtocolMessage(layer), cid]) +
                        FormatStatisticsTable(stats, layer);
        GetStatisticsTable := True;
    end;
end;

function FormatStatisticsTable(var table: LStatTableStruct; protocol: LProtocol): String;
var
    msgTable:   LMessageTable;
    report:     String;
    i:          LCount;
begin

    report      := Format('%12d : Bytes sent    '#13#10, [table.bytesSent]) +
                        Format('%12d : Bytes received'#13#10#13#10, [table.bytesRcvd]);
    msgTable    := lgo_StatisticMessages(protocol);

    for i := 0 to table.statCount - 1 do
    begin
        report := report + Format('%12d : %s'#13#10, [table.stat[i], msgTable^]);
        Inc(msgTable);
    end;

    FormatStatisticsTable := report;
end;

{******************************************************************************
                                 End of File
 ******************************************************************************}
end.