{******************************************************************************
 *                                                                            *
 *                                LayGO (tm)                                  *
 *           Copyright (c) 1998-2000 by Advanced Relay Corporation            *
 *                           All rights reserved.                             *
 *                                                                            *
 ******************************************************************************

  $Source: q:/laygorcs/Status.pas $

      $Id: Status.pas 1.2 2000/02/29 01:17:15 tiktin Release $

  EXTERNAL MODULE DESCRIPTION

    Exports routines for retreiving LayGO Status Reports.

 ******************************************************************************}

unit StatusReport;

interface

uses
    SysUtils, Laygo;

function FormatBoolean(b: Integer): String;
function FormatStatusReport(var report: LStatusReportStruct): String;
function FormatStatusSummary(var summary: LSummaryStruct): String;
function GetStatusReport(cid: LCid; var report: String): Boolean;

implementation

function GetStatusReport(cid: LCid; var report: String): Boolean;
var
    status: LStatusReportStruct;
    error:  LErrorCode;
begin

    error := lgo_GetStatus(cid, @status, SizeOf(LStatusReportStruct));

    if error < 0 then
    begin
        report := Format('Failure getting status report on CID %d: %s',
                            [cid, lgo_ErrorMessage(error)]);
        GetStatusReport := False;
    end
    else
    begin
        report := Format('Status Report for CID %d:'#13#10#13#10, [cid]) +
                            FormatStatusReport(status);
        GetStatusReport := True;
    end;
end;

function FormatStatusReport(var report: LStatusReportStruct): String;
var
    str:    string;
    i:      LCount;
begin

    for i := 0 to report.count - 1 do
    begin
        str := str + FormatStatusSummary(report.summary[i]);
    end;

    FormatStatusReport := str;
end;

function FormatStatusSummary(var summary: LSummaryStruct): String;
begin

    FormatStatusSummary := Format(
        'Status of layer: %s <%d,%d>'#13#10 +
        '   State:        %s'#13#10 +
        '   Blocked?:     %s'#13#10 +
        '   Blocking?:    %s'#13#10 +
        '   [Up %d] [Down %d] [Control %d]'#13#10#13#10,
        [ lgo_ProtocolMessage(summary.protocol),
        summary.major, summary.minor,
        lgo_StateMessage(summary.state),
        FormatBoolean(summary.isBlocked),
        FormatBoolean(summary.isBlocking),
        summary.upQueueCount,
        summary.downQueueCount,
        summary.controlQueueCount ]);
end;

function FormatBoolean(b: Integer): String;
begin

    if b <> 0 then
        FormatBoolean := 'True'
    else
        FormatBoolean := 'False';
end;

{******************************************************************************
                                 End of File
 ******************************************************************************}
end.