{******************************************************************************
 *                                                                            *
 *                                LayGO (tm)                                  *
 *           Copyright (c) 1998-2000 by Advanced Relay Corporation            *
 *                           All rights reserved.                             *
 *                                                                            *
 ******************************************************************************

  $Source: q:/laygorcs/Tdat.pas $

      $Id: Tdat.pas 1.2 2000/02/29 01:17:15 tiktin Release $

  EXTERNAL MODULE DESCRIPTION

    Exports the Test Data Frame routines.

 ******************************************************************************}

unit Tdat;

interface

uses
    SysUtils, Hex;

const

    TdatMax = 1023;

type

    TdatRecord = record
        name:   String;
        size:   SmallInt;
        flag:   SmallInt;
        frame:  HexPtr;
    end;

    TdatRecordPtr = ^TdatRecord;

var

    TdatSet:    array [0..TdatMax] of TdatRecord;
    TdatCount:  Integer;
    TdatFile:   String;

function TdatGetCount(): Integer;
function TdatGetFlag(i: Integer): Integer;
function TdatGetFrame(i: Integer): HexPtr;
function TdatGetLabel(i: Integer): String;
function TdatGetRecord(i: Integer): TdatRecordPtr;
function TdatGetSize(i: Integer): Integer;
function TdatLoadFile(fileName: String): Integer;

function IsComment(var str: String): Boolean;
function IsEndOfFile(str: String): Boolean;
function IsHeader(var str: String): Boolean;
function GetFlag(var str: String): Integer;
function GetDigit(str: String): Integer;
function GetBinDigit(str: String): Integer;
function GetHexDigit(str: String): Integer;

implementation

function TdatGetCount(): Integer;
begin
    TdatGetCount := TdatCount
end;

function TdatGetFlag(i: Integer): Integer;
begin
    if i < TdatCount then
        TdatGetFlag := TdatSet[i].flag
    else
        TdatGetFlag := 0;
end;

function TdatGetFrame(i: Integer): HexPtr;
begin
    if i < TdatCount then
        TdatGetFrame := TdatSet[i].frame
    else
        TdatGetFrame := nil;
end;

function TdatGetLabel(i: Integer): String;
begin
    if i < TdatCount then
        TdatGetLabel := TdatSet[i].name
    else
        TdatGetLabel := 'Invalid';
end;

function TdatGetRecord(i: Integer): TdatRecordPtr;
begin
    if i < TdatCount then
        TdatGetRecord := @TdatSet[i]
    else
        TdatGetRecord := nil;
end;

function TdatGetSize(i: Integer): Integer;
begin
    if i < TdatCount then
        TdatGetSize := TdatSet[i].size
    else
        TdatGetSize := 0;
end;

function TdatLoadFile(fileName: String): Integer;
var
    str:    String;
    tmp:    String;
    F:      TextFile;
    i:      Integer;
    flag:   Integer;
    tlabel: String;
    count:  Integer;
    quote:  String;
    n:      Integer;
    nmax:   Integer;
    p:      HexPtr;
begin

    quote   := Chr(34);
    count   := 0;
    flag    := 0;
    n       := 0;
    nmax    := 1024;
    p       := nil;

    if TdatCount > 0 then
    begin
        for i := 0 to TdatCount - 1 do
        begin
            FreeMem(TdatSet[i].frame);
        end;
        TdatCount   := 0;
        TdatFile    := '';
    end;

    try
        AssignFile(F, fileName);
        Reset(F);
        try
            while (count <= TdatMax) and (not Eof(F)) do
            begin
                Readln(F, str);
                if not IsComment(str) then
                begin
                    if IsEndOfFile(str) then
                    begin
                        Break;
                    end
                    else if IsHeader(str) then
                    begin
                        if count > 0 then
                        begin
                            { Save completed frame }
                            TdatSet[count - 1].name := tlabel;
                            TdatSet[count - 1].flag := flag;
                            TdatSet[count - 1].size := n;
                            ReallocMem(TdatSet[count - 1].frame, n);
                        end;

                        { Start new frame }
                        count   := count + 1;
                        n       := 0;
                        nmax    := 1024;

                        TdatSet[count - 1].frame := AllocMem(nmax);
                        p       := TdatSet[count - 1].frame;
                        flag    := GetFlag(str);
                        tlabel  := str;
                    end
                    else
                    begin
                        if count > 0 then
                        begin
                            while Length(str) > 0 do
                            begin
                                { Find the next token }
                                i := Pos(' ', str);
                                if i > 0 then
                                begin
                                    tmp := Copy(str, 1, i - 1);
                                    Delete(str, 1, i);
                                    str := Trim(str);
                                end
                                else
                                begin
                                    tmp := str;
                                    str := '';
                                end;

                                { Strip optional comma }
                                if tmp[Length(tmp)] = ',' then
                                    Delete(tmp, Length(tmp), 1);

                                { Quoted string of ASCII characters? }
                                if tmp[1] = quote then
                                begin
                                    Delete(tmp, 1, 1);
                                    { Strip close quote }
                                    if tmp[Length(tmp)] = quote then
                                        Delete(tmp, Length(tmp), 1);

                                    { Convert each character to byte }
                                    for i := 1 to Length(tmp) do
                                    begin
                                        if n = nmax then
                                        begin
                                            ReallocMem(TdatSet[count - 1].frame, nmax * 2);
                                            nmax    := nmax * 2;
                                            p       := Ptr(Integer(TdatSet[count - 1].frame) + n);
                                        end;

                                        p^  := Byte(tmp[i]);
                                        p   := Ptr(Integer(p) + 1);
                                        n   := n + 1;
                                    end;
                                end
                                else
                                begin
                                    { Hex or binary digit }
                                    if n = nmax then
                                    begin
                                        ReallocMem(TdatSet[count - 1].frame, nmax * 2);
                                        nmax    := nmax * 2;
                                        p       := Ptr(Integer(TdatSet[count - 1].frame) + n);
                                    end;

                                    p^  := Byte(GetDigit(tmp));
                                    p   := Ptr(Integer(p) + 1);
                                    n   := n + 1;
                                end;
                            end;
                        end;
                    end;
                end;
            end;
            if count > 0 then
            begin
                { Save completed frame }
                TdatSet[count - 1].name := tlabel;
                TdatSet[count - 1].flag := flag;
                TdatSet[count - 1].size := n;
                ReallocMem(TdatSet[count - 1].frame, n);
                { Save completed frame set }
                TdatCount   := count;
                TdatFile    := fileName;
            end;

            TdatLoadFile := TdatCount;
        except
            if count > 0 then
            begin
                for i := 0 to count - 1 do
                begin
                    FreeMem(TdatSet[i].frame);
                end;
            end;
            TdatLoadFile := 0;
        end;
        CloseFile(F);
    except
        TdatLoadFile := 0;
    end;
end;

function IsComment(var str: String): Boolean;
var
    i:  Integer;
begin

    IsComment := False;

    i := Pos('#', str);

    if i = 1 then
        IsComment := True
    else if i > 0 then
    begin
        Delete(str, i, Length(str) - i + 1);
        str := Trim(str);
        IsComment := Length(str) < 1;
    end;
end;

function IsEndOfFile(str: String): Boolean;
var
    i:  Integer;
begin

    IsEndOfFile := False;

    i := Pos(']', str);

    if i = 1 then
        IsEndOfFile := True;
end;

function IsHeader(var str: String): Boolean;
var
    i:  Integer;
begin

    IsHeader := False;

    i := Pos('[', str);

    if i = 1 then
    begin
        IsHeader := True;
        Delete(str, 1, 1);
        str := TrimLeft(str);
        i := Pos(']', str);
        if i > 0 then
        begin
            Delete(str, i, Length(str) - i + 1);
            str := TrimRight(str);
        end;
        if Length(str) < 1 then
            str := 'Unknown';
    end;
end;

function GetFlag(var str: String): Integer;
var
    i:      Integer;
    flag:   String;
begin

    GetFlag := 0;

    i := Pos(Chr(123), str);

    if i > 0 then
    begin
        flag := Copy(str, 1, i - 1);
        flag := Trim(flag);
        Delete(str, i, Length(str) - i + 1);
        str := Trim(str);
        if Length(flag) > 0 then
        begin
            i := Pos(Chr(125), flag);
            if i > 0 then
            begin
                Delete(flag, i, Length(flag) - i + 1);
                flag := Trim(flag);
            end;
            GetFlag := GetDigit(flag);
        end;
    end;
end;

function GetDigit(str: String): Integer;
var
    bin:    Boolean;
begin

    GetDigit := 0;
    bin      := False;

    str := Trim(str);

    if Length(str) > 0 then
    begin
        if (Length(str) > 1) and (str[1] = '0') then
        begin
            if str[2] = '%' then
            begin
                bin := True;
                Delete(str, 1, 2);
                str := Trim(str);
            end
            else if (str[2] = 'x') or (str[2] = 'X') then
            begin
                Delete(str, 1, 2);
                str := Trim(str);
            end;
        end;
        if bin then
            GetDigit := GetBinDigit(str)
        else
            GetDigit := GetHexDigit(str)
    end;
end;

function GetBinDigit(str: String): Integer;
var
    i:  Integer;
    n:  Integer;
begin

    n := 0;
    for i := 1 to Length(str) do
    begin
        n := n * 2;
        if str[i] = '1' then
            n := n + 1;
    end;

    GetBinDigit := n;
end;

function GetHexDigit(str: String): Integer;
var
    i:  Integer;
    n:  Integer;
begin

    n   := 0;
    str := UpperCase(str);

    for i := 1 to Length(str) do
    begin
        n := n * 16;
        case str[i] of
        '0' .. '9':
            n := n + StrToInt(str[i]);
        'A' .. 'F':
            n := n + (Ord(str[i]) - Ord('A')) + 10;
        else
            n := 0;
            Break;
        end;
    end;

    GetHexDigit := n;
end;

{******************************************************************************
                                 end of File
 ******************************************************************************}
end.

